@@ -64,7 +64,6 @@ typedef struct GifState {
     int stored_bg_color;
 
     GetByteContext gb;
-    /* LZW compatible decoder */
     LZWState *lzw;
 
     /* aux buffers */
@@ -140,11 +139,11 @@ static int gif_read_image(GifState *s, AVFrame *frame)
     if (bytestream2_get_bytes_left(&s->gb) < 9)
         return AVERROR_INVALIDDATA;
 
-    left = bytestream2_get_le16u(&s->gb);
-    top = bytestream2_get_le16u(&s->gb);
-    width = bytestream2_get_le16u(&s->gb);
+    left   = bytestream2_get_le16u(&s->gb);
+    top    = bytestream2_get_le16u(&s->gb);
+    width  = bytestream2_get_le16u(&s->gb);
     height = bytestream2_get_le16u(&s->gb);
-    flags = bytestream2_get_byteu(&s->gb);
+    flags  = bytestream2_get_byteu(&s->gb);
     is_interleaved = flags & 0x40;
     has_local_palette = flags & 0x80;
     bits_per_pixel = (flags & 0x07) + 1;
@@ -181,10 +180,14 @@ static int gif_read_image(GifState *s, AVFrame *frame)
 
     /* verify that all the image is inside the screen dimensions */
     if (left + width > s->screen_width ||
-        top + height > s->screen_height)
+        top + height > s->screen_height) {
+        av_log(s->avctx, AV_LOG_ERROR, "image is outside the screen dimensions.\n");
         return AVERROR_INVALIDDATA;
-    if (width <= 0 || height <= 0)
+    }
+    if (width <= 0 || height <= 0) {
+        av_log(s->avctx, AV_LOG_ERROR, "Invalid image dimensions.\n");
         return AVERROR_INVALIDDATA;
+    }
 
     /* process disposal method */
     if (s->gce_prev_disposal == GCE_DISPOSAL_BACKGROUND) {
@@ -234,8 +237,12 @@ static int gif_read_image(GifState *s, AVFrame *frame)
     pass = 0;
     y1 = 0;
     for (y = 0; y < height; y++) {
-        if (ff_lzw_decode(s->lzw, s->idx_line, width) == 0)
+        int count = ff_lzw_decode(s->lzw, s->idx_line, width);
+        if (count != width) {
+            if (count)
+                av_log(s->avctx, AV_LOG_ERROR, "LZW decode failed\n");
             goto decode_tail;
+        }
 
         pr = ptr + width;
 
@@ -251,21 +258,26 @@ static int gif_read_image(GifState *s, AVFrame *frame)
             case 1:
                 y1 += 8;
                 ptr += linesize * 8;
+                if (y1 >= height) {
+                    y1 = pass ? 2 : 4;
+                    ptr = ptr1 + linesize * y1;
+                    pass++;
+                }
                 break;
             case 2:
                 y1 += 4;
                 ptr += linesize * 4;
+                if (y1 >= height) {
+                    y1 = 1;
+                    ptr = ptr1 + linesize;
+                    pass++;
+                }
                 break;
             case 3:
                 y1 += 2;
                 ptr += linesize * 2;
                 break;
             }
-            while (y1 >= height) {
-                y1 = 4 >> pass;
-                ptr = ptr1 + linesize * y1;
-                pass++;
-            }
         } else {
             ptr += linesize;
         }
@@ -293,7 +305,7 @@ static int gif_read_extension(GifState *s)
         return AVERROR_INVALIDDATA;
 
     ext_code = bytestream2_get_byteu(&s->gb);
-    ext_len = bytestream2_get_byteu(&s->gb);
+    ext_len  = bytestream2_get_byteu(&s->gb);
 
     av_dlog(s->avctx, "ext_code=0x%x len=%d\n", ext_code, ext_len);
 
@@ -307,7 +319,7 @@ static int gif_read_extension(GifState *s)
         if (bytestream2_get_bytes_left(&s->gb) < 5)
             return AVERROR_INVALIDDATA;
 
-        gce_flags = bytestream2_get_byteu(&s->gb);
+        gce_flags    = bytestream2_get_byteu(&s->gb);
         bytestream2_skipu(&s->gb, 2);    // delay during which the frame is shown
         gce_transparent_index = bytestream2_get_byteu(&s->gb);
         if (gce_flags & 0x01)
@@ -361,7 +373,7 @@ static int gif_read_header1(GifState *s)
 
     /* read screen header */
     s->transparent_color_index = -1;
-    s->screen_width = bytestream2_get_le16u(&s->gb);
+    s->screen_width  = bytestream2_get_le16u(&s->gb);
     s->screen_height = bytestream2_get_le16u(&s->gb);
 
     v = bytestream2_get_byteu(&s->gb);
@@ -395,7 +407,7 @@ static int gif_read_header1(GifState *s)
 
 static int gif_parse_next_image(GifState *s, AVFrame *frame)
 {
-    while (bytestream2_get_bytes_left(&s->gb)) {
+    while (bytestream2_get_bytes_left(&s->gb) > 0) {
         int code = bytestream2_get_byte(&s->gb);
         int ret;
 
@@ -458,9 +470,8 @@ static int gif_decode_frame(AVCodecContext *avctx, void *data, int *got_frame, A
         if ((ret = gif_read_header1(s)) < 0)
             return ret;
 
-        if ((ret = av_image_check_size(s->screen_width, s->screen_height, 0, avctx)) < 0)
+        if ((ret = ff_set_dimensions(avctx, s->screen_width, s->screen_height)) < 0)
             return ret;
-        avcodec_set_dimensions(avctx, s->screen_width, s->screen_height);
 
         av_frame_unref(s->frame);
         if ((ret = ff_get_buffer(avctx, s->frame, 0)) < 0)
@@ -494,7 +505,7 @@ static int gif_decode_frame(AVCodecContext *avctx, void *data, int *got_frame, A
         return ret;
     *got_frame = 1;
 
-    return avpkt->size;
+    return bytestream2_tell(&s->gb);
 }
 
 static av_cold int gif_decode_close(AVCodecContext *avctx)
