@@ -143,7 +143,7 @@ int ff_get_cpu_flags_x86(void)
     if (max_std_level >= 7) {
         cpuid(7, eax, ebx, ecx, edx);
 #if HAVE_AVX2
-        if (ebx & 0x00000020)
+        if ((rval & AV_CPU_FLAG_AVX) && (ebx & 0x00000020))
             rval |= AV_CPU_FLAG_AVX2;
 #endif /* HAVE_AVX2 */
         /* BMI1/2 don't need OS support */
